@@ -1,5 +1,5 @@
 ;; match-window.jl -- match windows to properties
-;; $Id: match-window.jl,v 1.48 2001/04/20 21:40:39 jsh Exp $
+;; $Id: match-window.jl,v 1.49 2001/08/24 02:43:28 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -149,10 +149,8 @@
     :require sawfish.wm.ext.match-window)
 
   ;; used by sawmill-ui when grabbing property values
-  (define (match-window-grab-x-property prop-name)
-    (let ((real-prop (or (car (rassoc prop-name match-window-x-properties))
-			 (intern prop-name)))
-	  (window (select-window))
+  (define (match-window-grab-x-property real-prop)
+    (let ((window (select-window))
 	  prop)
       (when window
 	(setq prop (get-x-property window real-prop))
