@@ -1,5 +1,5 @@
 /* events.c -- Event handling
-   $Id: events.c,v 1.140 2001/08/25 01:16:42 jsh Exp $
+   $Id: events.c,v 1.143 2001/11/13 03:50:51 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -228,6 +228,12 @@ record_mouse_position (int x, int y, int event_type, Window w)
     }
 }
 
+void
+invalidate_cached_mouse_position (void)
+{
+    current_event_updated_mouse = FALSE;
+}
+
 static void
 install_colormaps (Lisp_Window *w)
 {
@@ -565,6 +571,10 @@ property_notify (XEvent *ev)
 		if (w == focus_window)
 		    install_colormaps (w);
 	    }
+	    else if (ev->xproperty.atom == xa_wm_protocols)
+	    {
+		get_window_protocols (w);
+	    }
 	}
 
 	rep_PUSHGC (gc_w, w_);
@@ -1278,7 +1288,7 @@ handle_input_mask(long mask)
 
 	record_event_time (&xev);
 	current_x_event = &xev;
-	current_event_updated_mouse = FALSE;
+	invalidate_cached_mouse_position ();
 	current_event_window = rep_NULL;
 
 	rep_PUSHGC(gc_old_current_window, old_current_window);
@@ -1292,6 +1302,7 @@ handle_input_mask(long mask)
     }
 
     emit_pending_destroys ();
+    commit_queued_focus_change ();
 }
 
 /* Handle all available X events on file descriptor FD. */
